import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.Hashtable;
import java.net.URL;
import java.net.MalformedURLException;
/**
 * SoundLoader osztly - hanganyagot tlt be a httrben.
 * @see java.lang.Thread
 * @see SoundList
 */
class SoundLoader extends Thread {
  SoundList soundList;                                      //letlts vezrlje
  String relativeURL;                                     //letltend fjl neve
  /**
   * Hanganyagot letlt programszl ltrehozsa
   * @param soundList a letltst vezrl hanganyaglista
   * @param relativeURL a letltend hanganyag relatv cme
   *                    a hanganyaglista bziscmhez kpest
   */
  public SoundLoader(SoundList soundList, String relativeURL) {
    this.soundList=soundList;
    this.relativeURL=relativeURL;
    start();                                     //httrben val futs indtsa
  }
  public void run() {                                          //httrben futs
    setPriority(MIN_PRIORITY);                             //senkit se zavarjunk
    soundList.applet.showStatus("Loading sound:"+soundList.baseURL+relativeURL);
    try {soundList.putClip(soundList.applet.getAudioClip(
                           soundList.baseURL, relativeURL), relativeURL);
    } catch (Exception e) {                                  //meg lett szaktva
      System.err.println("Can't load sound: "+soundList.baseURL+relativeURL);
    }
  }
}
/**SoundList osztly - hanganyagokat trol s azok letltst vezrli.
 * @see java.util.HashTable
 * @see SoundLoader
 */
class SoundList extends Hashtable {
  Applet applet;                                                  //a tulajdonos
  URL baseURL;                                     //hanganyag knyvtrnak cme
  /**Hanganyag trol s letltst vezrl ltrehozsa
   * @param applet a hanganyagot ignyl applet
   * @param baseURL a letltend hanganyagok bziscme
   */
  public SoundList(Applet applet, URL baseURL) {
    super(5);                                     //helyfoglals 5 klipp szmra
    this.applet = applet;
    this.baseURL = baseURL;
  }
  /**Hanganyag letltsnek megkezdse
   * @param relativeURL a letltend hanganyag relatv cme a bziscmhez kpest
   */
  public void startLoading(String relativeURL) {
    new SoundLoader(this, relativeURL);
  }
  /**Hanganyag lekrdezse
   * @param relativeURL a letltend hanganyag relatv cme a bziscmhez kpest
   * @result null, ha mg nincs betltve a hanganyag,
   *         klnben a hanganyagot tartalmaz AudioClip
   * @see java.applet.AudioClip
   */
  public AudioClip getClip(String relativeURL) {
    AudioClip clip=(AudioClip)get(relativeURL);
    if (clip==null) applet.showStatus("Sound: "+baseURL+relativeURL+" loading");
    else applet.showStatus("");
    return clip;
  }
  /**Hanganyag eltrolsa
   * @param clip az eltroland hanganyag
   * @param relativeURL a letltend hanganyag relatv cme a bziscmhez kpest
   * @see java.applet.AudioClip
   */
  public void putClip(AudioClip clip, String relativeURL) {
    put(relativeURL, clip);
  }
}
/**
 * GraphicsButton osztly - kpet is megjelent nyomgomb
 * @see java.awt.Canvas
 * @see BrowserTest
 */
class GraphicsButton extends Canvas
                     implements FocusListener, KeyListener, MouseListener {
  private BrowserTest applet;                                            //szl
  private boolean focused=false;                             //nla van a fkusz
  private boolean pressed=false;                                 //be van nyomva
  Image picture;                                                     //httrkp
  String text;                                                         //felirat
  URL link;                                                //megjelentend link
  String linkFrame;                                      //megjelents frame-je
  String statusText;                                         //grgetend szveg
  String soundName;                                        //httrzene fjlneve
  String clickSoundName;                            //kivlasztsi zene fjlneve
  SoundList soundList;                                  //zenebetlts vezrlje
  AudioClip sound;                                                  //httrzene
  AudioClip clickSound;                                      //kivlasztsi zene
  Image offscreen;                                               //offscreen kp
  String actionCommand;                      //lenyomskor kldtt parancsszveg
  transient ActionListener actionListener;        //gombnyomst figyelk listja
  /**
   * Grafikus nyomgomb ltrehozsa
   * @param applet a tulajdonos BrowserTest applet
   * @param img a gomb httrkpe
   * @param text a gomb felirata
   */
  public GraphicsButton(BrowserTest applet, Image img, String text) {
    picture=img;
    this.applet=applet;
    this.text=text;
    addFocusListener(this);                                //esemnyek figyelse
    addKeyListener(this);
    addMouseListener(this);
    requestFocus();
  }
  public void invalidate() {
    super.invalidate();
    offscreen=null;                                  //offscreen buffer eldobsa
  }
  public Dimension getPreferredSize() {                  //sajt mret megrzse
    if (picture!=null &&                    //ha van httrkp, akkor van mrete
       (checkImage(picture, this)&(WIDTH|HEIGHT))==(WIDTH|HEIGHT))
       return new Dimension(
                  picture.getWidth(this)+10, picture.getHeight(this)+10);
    Font f=getFont();
    if (f!=null) {int w=0;
       FontMetrics fm = getFontMetrics(f);
       if (text!=null) w=fm.stringWidth(text);
       return new Dimension(w+10, fm.getHeight()+fm.getAscent()+5);
    }
    return getMinimumSize();
  }
  public Dimension getMinimumSize() {                    //alaprtelmezett mret
    return new Dimension(10, 10);
  }
  public void update(Graphics g) {                                  //kirajzols
    paint(g);
  }
  public void paint(Graphics g) {                                   //kirajzols
    if (getSize().width==0 || getSize().height==0) return;
    int sx, sy;
    if (offscreen==null)                          //offscreen buffer ltrehozsa
       offscreen=createImage(getSize().width, getSize().height);
    Graphics og=offscreen.getGraphics();                    //offscreen rajzols
    if (focused) og.setColor(getForeground());             //aktulis kr keret
    else og.setColor(getBackground());
    og.drawRect(0, 0, getSize().width-1, getSize().height-1);
    og.setColor(getForeground());
    og.drawRect(1, 1, getSize().width-3, getSize().height-3);
    if (pressed) sx=sy=1;
    else sx=sy=0;
    if ((picture!=null)&&((checkImage(picture, this)&ERROR)==ERROR))
       picture=null;                                 //httrkp nem tlthet be
    og.draw3DRect(2, 2, getSize().width-5, getSize().height-5, !pressed);
    og.setColor(getBackground());
    og.fill3DRect(3, 3, getSize().width-6, getSize().height-6, !pressed);
    if (picture!=null)                                   //httrkp kirajzolsa
      og.drawImage(picture,sx+3,sy+3,getSize().width-6,getSize().height-6,this);
    if (text!=null) {                                //felirat kzpre rajzolsa
       FontMetrics fm=g.getFontMetrics();
       if (fm!=null) {
          og.setColor(getForeground());
          sx+=(int)(getSize().width-fm.stringWidth(text))/2;
          sy+=(int)(getSize().height-fm.getHeight())/2+fm.getAscent();
          og.drawString(text, sx, sy);
       }
    }
    g.drawImage(offscreen, 0, 0, null);              //offscreen kp kirajzolsa
    og.dispose();
  }
  /**Lenyomskor kldtt parancsszveg belltsa
   * Ha nincs megadva, a gomb felirata lesz elkldve
   */
  public void setActionCommand(String command) {
    actionCommand=command;
  }
  /**
   * Lenyomskor kldtt parancsszveg lekrdezse
   */
  public String getActionCommand() {
    return (actionCommand==null ? text : actionCommand);
  }
  /**
   * Megnyomsfigyel listba vtele
   * @param l a megnyoms figyelje
   */
  public void addActionListener(ActionListener l) {
    actionListener=AWTEventMulticaster.add(actionListener, l);
  }
  /**
   * Megnyomsfigyel listbl trlse
   * @param l a megnyoms figyelje
   */
  public void removeActionListener(ActionListener l) {
    actionListener=AWTEventMulticaster.remove(actionListener, l);
  }
  protected void processAction() {                        //megnyoms lekezelse
    if (clickSound==null && clickSoundName!=null)          //zene be van tltve?
       clickSound=soundList.getClip(clickSoundName);
    if (clickSound!=null) clickSound.play();                           //zenls
    if (actionListener!=null)                    //gombnyomsfigyelk rtestse
       actionListener.actionPerformed(
         new ActionEvent(this,ActionEvent.ACTION_PERFORMED,getActionCommand()));
    pressed=false;
    repaint();
  }
  public void focusGained(FocusEvent e) {                   //megkapta a fkuszt
    focused=true;
    repaint();
  }
  public void focusLost(FocusEvent e) {                  //elvesztette a fkuszt
    pressed=focused=false;
    repaint();
  }
  public void keyTyped(KeyEvent e) {};
  public void keyPressed(KeyEvent e) {
    if ((e.getKeyCode()==KeyEvent.VK_SPACE)&&!pressed) {  //megnyomtk a szkzt
       pressed=true;
       repaint();
    }
  }
  public void keyReleased(KeyEvent e) {
    if ((e.getKeyCode()==KeyEvent.VK_SPACE)&&pressed)     //megnyomtk a szkzt
       processAction();
  }
  public void mouseEntered(MouseEvent e) {                        //kurzor belp
    if (sound==null && soundName!=null)                   //httrzene betltve?
       sound=soundList.getClip(soundName);
    if (sound!=null) sound.loop();                               //httrzenls
    applet.scrollStatus(statusText);                   //sttuszszveg grgetse
    if (link!=null) applet.showStatus(link+"");                     //linkkirs
  }
  public void mouseExited(MouseEvent e) {                         //kurzor kilp
    if (pressed) {                                          //mr nincs benyomva
       pressed=false;
       repaint();
    }
    if (sound!=null) sound.stop();                            //httrzene lell
    applet.scrollStatus(null);                                  //grgets lell
  }
  public void mousePressed(MouseEvent e) {                  //egrgomb megnyomva
    if (!pressed) {
       pressed=true;
       requestFocus(); repaint();
    }
  }
  public void mouseReleased(MouseEvent e) {                 //egrgomb elengedve
    if (pressed) processAction();
  }
  public void mouseClicked(MouseEvent e) {};
}
/**
 * BrowserTest osztly - HTML bngsz program
 *                        hipermdia kpessgeit szemlltet applet
 * @see java.applet.Applet
 */
public class BrowserTest extends Applet
                         implements Runnable, ActionListener, MouseListener {
  URL imageSource=null;                                //kpek knyvtrnak cme
  URL soundSource=null;                               //hangok knyvtrnak cme
  SoundList soundList;                                    //hangbetlts vezrl
  Image background;                                                 //httr kp
  AudioClip sound;                                                 //httr zene
  String soundName;                                  //httr hanganyag fjlneve
  private boolean loaded;                     //paramter-feldolgozs befejezve?
  private int count=1;                                       //gombok szmllja
  private Thread kicker;                                          //sajt Thread
  int statusDelay=100;                    //sttuszsor grgetsnek ksleltetse
  String statusText=null;                                    //grgetend szveg
  private StringBuffer displayText=new StringBuffer();  //aktulis sttuszszveg
  /**
   * Sznparamter feldolgozsa
   * @param szin a sznt reprezentl szveg
   * @result a krt szn, vagy null, ha a paramter helytelen
   */
  Color decodeColor(String szin) {
    int val=0;
    try {
        if (szin.startsWith("0x")) {                       //hexban van megadva
           val=Integer.parseInt(szin.substring(2), 16);
        } else if (szin.startsWith("#")) {                 //hexban van megadva
               val=Integer.parseInt(szin.substring(1), 16);
        } else if (szin.startsWith("0") && szin.length()>1) {          //oktlis
               val=Integer.parseInt(szin.substring(1), 8);
        } else val=Integer.parseInt(szin, 10);         //decimlisan van megadva
        return new Color(val);
    } catch (NumberFormatException e) {return null;}
  }
  /**Paramterekrl informci
   */
  public String[][] getParameterInfo() {
    String info[][]={
       {"foregroundcolor", "szn", "alaprtelmezett eltrszn"},
       {"backgroundcolor", "szn", "alaprtelmezett httrszn"},
       {"imagesource", "URL", "alaprtelmezett relatv URL a kpekhez"},
       {"soundsource", "URL", "alaprtelmezett relatv URL a hangokhoz"},
       {"background", "kpfjl", "httrkp"},
       {"sound", "hangfjl", "httrzene"},
       {"statusdelay", "int", "grgets vrakozsi ideje sec-ban"},
       {"textX", "szveg", "X. gomb felirata"},
       {"imageX", "kpfjl", "X. gomb kpe"},
       {"soundX", "hangfjl", "X. gomb httrzenje"},
       {"clicksoundX", "hangfjl", "X. gomb kivlasztskori zenje"},
       {"foregroundcolorX", "szn", "X. gomb eltrszne"},
       {"backgroundcolorX", "szn", "X. gomb httrszne"},
       {"linkX", "URL", "X. gomb megjelentend linkje"},
       {"rellinkX", "URL", "X. gomb megjelentend relatv linkje"},
       {"linkframeX", "frame", "X. gomb linkjnek frame-je"},
       {"statustextX", "szveg", "X. gomb grgetett sttuszszvege"}
    };
    return info;
  }
  /**
   * Appletrl informci
   */
  public String getAppletInfo() {
    return "Az applet a bngsz program hipermdia kpessgeit szemllteti.";
  }
  public void init() {                                           //inicializls
    Color color;

    String param=getParameter("imageSource");    //globlis paramterek tvtele
    try {imageSource=(param==null) ? getDocumentBase() :
                                     new URL(getDocumentBase(), param+"/");
    } catch (MalformedURLException e) {
      System.err.println("BrowserTest imageSource: "+e+" hibs."+
                         "Helyette a DocumentBase lesz figyelembe vve");
      imageSource=getDocumentBase();
    }
    param=getParameter("soundSource");                             //imagesource
    try {soundSource=(param==null) ? getDocumentBase() :
                                     new URL(getDocumentBase(), param+"/");
    } catch (MalformedURLException e) {
      System.err.println("BrowserTest soundSource: "+e+" hibs."+
                         "Helyette a DocumentBase lesz figyelembe vve");
      soundSource=getDocumentBase();
    }
    if ((param=getParameter("background"))!=null) {                 //background
       try {background=getImage(new URL(imageSource, param));
       } catch (Exception e) {System.err.println(this+": "+e);}
    }
    if ((param=getParameter("backgroundColor"))!=null &&       //backgroundcolor
       ((color=decodeColor(param))!=null))
       setBackground(color);
    if ((param=getParameter("foregroundColor"))!=null &&       //foregroundcolor
       ((color=decodeColor(param))!=null))
       setForeground(color);
    soundList=new SoundList(this, soundSource);
    if ((soundName=getParameter("sound"))!=null)                         //sound
       soundList.startLoading(soundName);                     //zenetlts indul
    if ((param=getParameter("statusDelay"))!=null) {               //statusdelay
       try {statusDelay=Integer.parseInt(param);
       } catch (Exception e) {}
    }
    if (statusDelay<100) statusDelay=100;
    addMouseListener(this);
  }
  public void start() {                                                //indts
    if (kicker==null) {                                  //sajt Thread indtsa
       kicker=new Thread(this);
       kicker.setPriority(Thread.MIN_PRIORITY+1);
       kicker.start();
    }
  }
  public void run() {                             //sajt Thread httrbe futsa
    String text;
    String image;
    String param;
    Image img;
    GraphicsButton gb;
    Color color;

    if (!loaded) {                                    //mg paramterfeldolgozs
       while (((text=getParameter("text"+count))!=null) |
            ((image=getParameter("image"+count))!=null)) {
            img=null;
            if (image!=null)                                  //kp feldolgozsa
               try {prepareImage(img=getImage(imageSource,image), this);
                   while (((checkImage(img, this) & ERROR)!=ERROR) &&//vrakozs
                        ((checkImage(img,this)&(WIDTH|HEIGHT))!=(WIDTH|HEIGHT)))
                        try {Thread.currentThread().sleep(100);
                        } catch (Exception e) {}
               } catch (Exception e) {}
            gb=new GraphicsButton(this, img, text);        //j gomb hozzvtele
            if ((param=getParameter("link"+count))!=null)                //linkX
               try {gb.link=new URL(param);
               } catch (Exception e) {
                 System.err.println(this+" link: "+e);
               }
            if ((param=getParameter("rellink"+count))!=null)          //rellinkX
               try {gb.link=new URL(getDocumentBase(), param);
               } catch (Exception e) {
                 System.err.println(this+" rellink: "+e);
               }
            gb.linkFrame=getParameter("linkframe"+count);           //linkframeX
            gb.statusText=getParameter("statustext"+count);        //statustextX
            if ((gb.soundName=getParameter("sound"+count))!=null)       //soundX
               soundList.startLoading(gb.soundName);          //zenetlts indul
            if ((gb.clickSoundName=getParameter("clicksound"+count))!=null)
               soundList.startLoading(gb.clickSoundName);       //letlts indul
            gb.soundList=soundList;
            color=null;
            if ((param=getParameter("backgroundColor"+count))!=null)
               color=decodeColor(param);                      //backgroundcolorX
            if (color==null) color=getBackground();
            gb.setBackground(color);
            color=null;
            if ((param=getParameter("foregroundColor"+count))!=null)
               color=decodeColor(param);                      //foregroundcolorX
            if (color==null) color=getForeground();
            gb.setForeground(color);
            add(gb);
            gb.addActionListener(this);
            validate();
            count++;                       //jhet a kvetkez gomb feldolgozsa
            if (Thread.currentThread()!=kicker) {       //thread le lett lltva
               Thread.currentThread().stop();
               return;
            }
        }
        loaded=true;                            //paramterfeldolgozs befejezve
    }
    displayText.setLength(0);
    while (Thread.currentThread()==kicker) {            //amg aktulis a Thread
          scroll();                                                   //grgets
          try {Thread.currentThread().sleep(statusDelay);            //vrakozs
          } catch (Exception e) {}
    }
  }
  private synchronized void scroll() {                  //sttuszsorban grgets
    if (statusText!=null) {                                   //van mit grgetni
       if (displayText.length()==0) {                            //mg nem grg
          displayText.ensureCapacity(statusText.length());
          displayText.setLength(0);
          displayText.insert(0, statusText);
       }
       showStatus(displayText.toString());                              //kirs
       if (displayText.length()>0) {                                  //lptets
          for (int i=1; i<displayText.length(); i++)
             displayText.setCharAt(i-1,displayText.charAt(i));
          displayText.setLength(displayText.length()-1);
       }
    }
  }
  public void stop() {                                              //megllts
    if (kicker!=null) {
       kicker.stop();                                 //sajt Thread meglltsa
       kicker=null;
    }
  }

  /**
   * j szveg grgetse
   */
  public synchronized void scrollStatus(String text) {
    statusText=text;
    showStatus("");
    displayText.setLength(0);
  }
  public void update(Graphics g) {
      paint(g);
  }
  public void paint(Graphics g) {                                   //kirajzols
    if (background!=null)                               //httrkp megrajzolsa
       g.drawImage(background, 0, 0, getSize().width, getSize().height, this);
  }
  public void mouseEntered(MouseEvent evt) {                      //kurzor belp
    if (sound==null && soundName!=null)                   //httrzene betltve?
       sound=soundList.getClip(soundName);
    if (sound!=null) sound.loop();                         //httrzene indtsa
  }
  public void mouseExited(MouseEvent evt) {                       //kurzor kilp
    if (sound!=null) sound.stop();                       //httrzene lelltsa
  }
  public void mousePressed(MouseEvent evt) {};                   //MouseListener
  public void mouseReleased(MouseEvent evt) {};                      //interfsz
  public void mouseClicked(MouseEvent evt) {};                  //miatt kellenek
  public void actionPerformed(ActionEvent evt){                  //esemnykezel
    GraphicsButton gb=(GraphicsButton)evt.getSource();
    if (gb.link!=null) {                              //adott link megjelentse
       if (gb.linkFrame==null) getAppletContext().showDocument(gb.link);
       else getAppletContext().showDocument(gb.link, gb.linkFrame);
    }
  }
}
